import { Provider } from '@nestjs/common';
import { PubSubAsyncConfig } from './interfaces/pub-sub-async-config.interface';
import { PubSubOptions } from './interfaces/pub-sub-options.interface';
import { PubSub } from './pub-sub';
import { getPubSubConfigToken, getPubSubToken } from './utils/token';

export function createPubSubProvider(name: string): Provider {
  return {
    provide: getPubSubToken(name),
    useFactory: (option: PubSubOptions) => new PubSub(option),
    inject: [getPubSubConfigToken(name)],
  };
}

export function createOptionsProvider(config: PubSubAsyncConfig): Provider {
  return {
    provide: getPubSubConfigToken(config.name),
    useFactory: config.useFactory,
    inject: config.inject || [],
  };
}
export function createAsyncPubSubProviders(
  configs: PubSubAsyncConfig[],
): Provider[] {
  return configs
    .map((config) => [
      createOptionsProvider(config),
      createPubSubProvider(config.name),
    ])
    .flat();
}
